Imports System ' for IntPtr
Imports System.Runtime.InteropServices
Imports System.Text ' for StringBuilder

Namespace De.Tichnut.JewishCalendarZmanimDLL

  '******** Error Codes
  Public Class JEWCALDLLErrorCode
    Public Const JEWCAL_ERROR_OK                  As Integer = 0
    Public Const JEWCAL_ERROR_NULL_POINTER        As Integer = 1
    Public Const JEWCAL_ERROR_INVALID_STRUCT_SIZE As Integer = 2
    Public Const JEWCAL_ERROR_BUFFERTOOSMALL      As Integer = 3
    Public Const JEWCAL_ERROR_INVALID_FLAGS       As Integer = 4
    Public Const JEWCAL_ERROR_FILE_NOT_FOUND      As Integer = 5

    Public Const JEWCAL_ERROR_INVALID_GREGORIAN_DAY   As Integer = 100
    Public Const JEWCAL_ERROR_INVALID_GREGORIAN_MONTH As Integer = 101
    Public Const JEWCAL_ERROR_INVALID_GREGORIAN_YEAR  As Integer = 102
    Public Const JEWCAL_ERROR_INVALID_HEBREW_MONTH    As Integer = 103
    Public Const JEWCAL_ERROR_INVALID_HEBREW_YEAR     As Integer = 104
    Public Const JEWCAL_ERROR_NO_SHABBAT              As Integer = 105
    Public Const JEWCAL_ERROR_INVALID_DST             As Integer = 106
    Public Const JEWCAL_ERROR_INVALID_HEBREW_DAY      As Integer = 107
    Public Const JEWCAL_ERROR_NO_OMER_COUNT           As Integer = 108

    Public Const JEWCAL_ERROR_CITY_NOT_FOUND    As Integer = 200
    Public Const JEWCAL_ERROR_INVALID_LATITUDE  As Integer = 201
    Public Const JEWCAL_ERROR_INVALID_LONGITUDE As Integer = 202
    Public Const JEWCAL_ERROR_INVALID_TIMEZONE  As Integer = 203

    Public Const JEWCAL_ERROR_CALCULATE_ZMANIM_FAILED  As Integer = 300
    Public Const JEWCAL_ERROR_PARASHGETREADINGS_FAILED As Integer = 301
    Public Const JEWCAL_ERROR_NO_ASTRONOMICAL_EVENT    As Integer = 302
  End Class

  '******** Data structures

  '-------- Gregorian and Hebrew dates
  <StructLayout(LayoutKind.Sequential, Pack:=1)> _
  Public Structure JEWCALDATE
    Public Day As Integer
    Public Month As Integer
    Public Year As Integer
  End Structure

  '-------- Jewish holidays
  <StructLayout(LayoutKind.Sequential, Pack:=1, CharSet:=CharSet.Ansi)> _
  Public Structure JEWCALGETHOLIDAYINFO
    ' SAMPLE OF USAGE:
    '   Dim HolidayInfo As JEWCALGETHOLIDAYINFO
    '   HolidayInfo.cbSize = Marshal.SizeOf(HolidayInfo)
    Public cbSize As Integer

    ' SAMPLE OF USAGE:
    '   Dim gDate As JEWCALDATE
    '   - Set gDate members -
    '   Dim HolidayInfo As JEWCALGETHOLIDAYINFO
    '   HolidayInfo.GregorianDate = Marshal.AllocCoTaskMem(Marshal.SizeOf(gDate))
    '   Marshal.StructureToPtr(gDate, HolidayInfo.GregorianDate, False)
    Public GregorianDate As IntPtr

    Public IsDiaspora As Boolean
    Public PostponeShushanPurim As Boolean

    ' SAMPLE OF USAGE:
    '   Dim HolidayInfo As JEWCALGETHOLIDAYINFO
    '   HolidayInfo.cbMax = 256
    '   HolidayInfo.ReturnString = New String(" ", HolidayInfo.cbMax)
    Public ReturnString As String

    Public cbMax As Integer
    Public Language As String
    Public NameFile As String

    Public DayOfHoliday As Integer
    Public WeekdayOfHoliday As Integer
  End Structure

  '-------- Torah and haphtarah readings
  <StructLayout(LayoutKind.Sequential, Pack:=1, CharSet:=CharSet.Ansi)> _
  Public Structure JEWCALGETREADINGINFO
    ' SAMPLE OF USAGE:
    '   Dim ReadingInfo As JEWCALGETREADINGINFO
    '   ReadingInfo.cbSize = Marshal.SizeOf(ReadingInfo)
    Public cbSize As Integer

    ' SAMPLE OF USAGE:
    '   Dim gDate As JEWCALDATE
    '   - Set gDate members -
    '   Dim ReadingInfo As JEWCALGETREADINGINFO
    '   ReadingInfo.GregorianDate = Marshal.AllocCoTaskMem(Marshal.SizeOf(gDate))
    '   Marshal.StructureToPtr(gDate, ReadingInfo.GregorianDate, False)
    Public GregorianDate As IntPtr

    Public IsDiaspora As Boolean

    ' SAMPLE OF USAGE:
    '   Dim ReadingInfo As JEWCALGETREADINGINFO
    '   ReadingInfo.cbParashaMax = 256
    '   ReadingInfo.Parasha = New String(" ", ReadingInfo.cbParashaMax)
    Public Parasha As String
    Public cbParashaMax As Integer

    ' SAMPLE OF USAGE:
    '   Dim ReadingInfo As JEWCALGETREADINGINFO
    '   ReadingInfo.cbHaphtarahMax = 256
    '   ReadingInfo.Haphtarah = New String(" ", ReadingInfo.cbHaphtarahMax)
    Public Haphtarah As String
    Public cbHaphtarahMax As Integer

    Public Language As String
    Public NameFile As String

    Public IsParashaConnected As Boolean
  End Structure

  '-------- Shabbat times and Zmanim calculation
  <StructLayout(LayoutKind.Sequential, Pack:=1)> _
  Public Structure JEWCALDST
    Public StartWeekNo As Integer
    Public StartWeekday As Integer
    Public StartMonth As Integer

    Public EndWeekNo As Integer
    Public EndWeekday As Integer
    Public EndMonth As Integer
  End Structure

  <StructLayout(LayoutKind.Sequential, Pack:=1, CharSet:=CharSet.Ansi)> _
  Public Structure JEWCALTIME
    ' SAMPLE OF USAGE:
    '   Dim CalTime As JEWCALTIME
    '   CalTime.cbSize = Marshal.SizeOf(CalTime)
    Public cbSize As Integer

    Public Flags As Integer

    Public CityFile As String
    Public City As String

    Public LatitudeType As Byte
    Public LatitudeDeg As Integer
    Public LatitudeMin As Integer
    Public LongitudeType As Byte
    Public LongitudeDeg As Integer
    Public LongitudeMin As Integer
    Public TimeZone As Integer

    Public Calculation As String

    Public TimeFileName As String
    Public TimeInFile As String

    Public DateToCalculate As JEWCALDATE

    ' SAMPLE OF USAGE:
    '   Enable DST
    '     Dim dst As JEWCALDST
    '     - Set dst members -
    '     Dim CalTime As JEWCALTIME
    '     CalTime.DST = Marshal.AllocCoTaskMem(Marshal.SizeOf(dst))
    '     Marshal.StructureToPtr(dst, CalTime.DST, False)
    '   Disable DST
    '     Dim CalTime As JEWCALTIME
    '     CalTime.DST = IntPtr.Zero
    Public DST As IntPtr

    Public ReturnHour As Integer
    Public ReturnMin As Integer
  End Structure

  '******** API of JEWCAL.DLL
  Public Class JEWCALDLLAPI
    Public Const JEWCAL_WEEKDAY_SUNDAY    As Integer = 0
    Public Const JEWCAL_WEEKDAY_MONDAY    As Integer = 1
    Public Const JEWCAL_WEEKDAY_TUESDAY   As Integer = 2
    Public Const JEWCAL_WEEKDAY_WEDNESDAY As Integer = 3
    Public Const JEWCAL_WEEKDAY_THURSDAY  As Integer = 4
    Public Const JEWCAL_WEEKDAY_FRIDAY    As Integer = 5
    Public Const JEWCAL_WEEKDAY_SATURDAY  As Integer = 6

    '-------- Gregorian and Hebrew dates
    Declare Function JewcalGregorianDateToHebrewDate Lib "JEWCAL.DLL" _
          Alias "_JewcalGregorianDateToHebrewDate@8" _
          (ByRef GregorianDate As JEWCALDATE, _
           ByRef HebrewDate As JEWCALDATE) As Integer

    Declare Function JewcalHebrewDateToGregorianDate Lib "JEWCAL.DLL" _
          Alias "_JewcalHebrewDateToGregorianDate@8" _
          (ByRef HebrewDate As JEWCALDATE, _
           ByRef GregorianDate As JEWCALDATE) As Integer

    ' NOTE: Provide a StringBuilder for ReturnString which has a
    ' capacity of cbMax characters, e.g.
    '   Dim Result As StringBuilder
    '   Result = New StringBuilder(256)
    Declare Function JewcalGetHebrewMonthName Lib "JEWCAL.DLL" _
           Alias "_JewcalGetHebrewMonthNameA@24" _
           (Month As Integer, Year As Integer, _
            ReturnString As StringBuilder, cbMax As Integer, _
            Language As String, _
            NameFile As String) As Integer

    '-------- Finding Shabbat dates
    Declare Function JewcalGetNearestShabbatGregorianDate Lib "JEWCAL.DLL" _
           Alias "_JewcalGetNearestShabbatGregorianDate@8" _
           (ByRef GregorianDate As JEWCALDATE, _
            ByRef ReturnGregorianDate As JEWCALDATE) As Integer

    Declare Function JewcalGetNearestShabbatHebrewDate Lib "JEWCAL.DLL" _
           Alias "_JewcalGetNearestShabbatHebrewDate@8" _
           (ByRef HebrewDate As JEWCALDATE, _
            ByRef ReturnHebrewDate As JEWCALDATE) As Integer

    '-------- Finding Holiday dates
    Public Const JEWCAL_DAY_HOLIDAY      As Integer = 1
    Public Const JEWCAL_DAY_YOMHAZICARON As Integer = 2
    Public Const JEWCAL_DAY_FAST         As Integer = 3

    Declare Function JewcalGetNearestHolidayGregorianDate Lib "JEWCAL.DLL" _
           Alias "_JewcalGetNearestHolidayGregorianDateA@24" _
           (IsDiaspora As Boolean, PostponeShushanPurim As Boolean, _
            NameFile As String, _
            ByRef GregorianDate As JEWCALDATE, _
            ByRef ReturnGregorianDate As JEWCALDATE, _
            ReturnHoliday As Integer) As Integer

    Declare Function JewcalGetNearestHolidayHebrewDate Lib "JEWCAL.DLL" _
           Alias "_JewcalGetNearestHolidayHebrewDateA@24" _
           (IsDiaspora As Boolean, PostponeShushanPurim As Boolean, _
            NameFile As String, _
            ByRef HebrewDate As JEWCALDATE, _
            ByRef ReturnHebrewDate As JEWCALDATE, _
            ReturnHoliday As Integer) As Integer

    '-------- Jewish holidays
    ' NOTE: Provide a StringBuilder for ReturnString which has a
    ' capacity of cbMax characters, e.g.
    '   Dim Result As StringBuilder
    '   Result = New StringBuilder(256)
    Declare Function JewcalGetJewishHolidayName Lib "JEWCAL.DLL" _
           Alias "_JewcalGetJewishHolidayNameA@28" _
           (ByRef GregorianDate As JEWCALDATE, _
            IsDiaspora As Boolean, PostponeShushanPurim As Boolean, _
            ReturnString As StringBuilder, cbMax As Integer, _
            Language As String, NameFile As String) As Integer

    Declare Function JewcalGetJewishHolidayNameEx Lib "JEWCAL.DLL" _
           Alias "_JewcalGetJewishHolidayNameExA@4" _
           (ByRef GetHolidayInfo As JEWCALGETHOLIDAYINFO) As Integer

    '-------- Omer count
    Declare Function JewcalGetOmerCountHebrewDate Lib "JEWCAL.DLL" _
           Alias "_JewcalGetOmerCountHebrewDate@8" _
           (ByRef HebrewDate As JEWCALDATE, _
            ByRef OmerCount As Integer) As Integer

    '-------- Torah and haphtarah readings
    Declare Function JewcalGetReadingEx Lib "JEWCAL.DLL" _
           Alias "_JewcalGetReadingExA@4" _
           (ByRef ReadingInfo As JEWCALGETREADINGINFO) As Integer

    '-------- Shabbat times and Zmanim calculation
    Public Const DST_WEEK_LAST         As Integer = 5

    Public Const DST_WEEKDAY_SUNDAY    As Integer = 0
    Public Const DST_WEEKDAY_MONDAY    As Integer = 1
    Public Const DST_WEEKDAY_TUESDAY   As Integer = 2
    Public Const DST_WEEKDAY_WEDNESDAY As Integer = 3
    Public Const DST_WEEKDAY_THURSDAY  As Integer = 4
    Public Const DST_WEEKDAY_FRIDAY    As Integer = 5
    Public Const DST_WEEKDAY_SATURDAY  As Integer = 6

    Public Const JEWCALTIME_CITY       As Integer = &H00000001
    Public Const JEWCALTIME_COORDS     As Integer = &H00000002

    Public Const JEWCALTIME_CALCDIRECT As Integer = &H00000004
    Public Const JEWCALTIME_CALCFILE   As Integer = &H00000008

    Public Const LATITUDE_NORTH As Byte = &H4E ' N
    Public Const LATITUDE_SOUTH As Byte = &H53 ' S
    Public Const LONGITUDE_EAST As Byte = &H45 ' E
    Public Const LONGITUDE_WEST As Byte = &H57 ' W

    Declare Function JewcalCalculateTime Lib "JEWCAL.DLL" _
           Alias "_JewcalCalculateTimeA@4" _
           (ByRef CalTime As JEWCALTIME) As Integer
  End Class

End Namespace
